using System;
using System.Collections.Generic;
using System.IO;
using System.Net.Sockets;

using Smdn.Protocols.Imap4.Protocol;

namespace Smdn.Protocols.Imap4.Session {
  // Layers:
  //   Client.ImapClient
  //   Client.ImapClientAgent
  //   Session.ImapSessionAgent
  // * Session.ImapSession
  //   Transaction.ImapTransactionAgent
  //   Transaction.ImapTransaction/Transaction.ImapRequest/Transaction.ImapResponse
  //   Protocol.ImapConnection/Protocol.ImapProtocolCommand/Protocol.ImapProtocolResponse
  //   (IMAP4)

  public class ImapSession {
    internal ImapConnection Connection {
      get { return connection; }
    }

    internal int Id {
      get { return id; }
    }

    public ImapServerInfo ServerInfo {
      get { return connection.ServerInfo; }
    }

    public ImapUri HostUri {
      get { return connection.ServerInfo.Uri; }
    }

    public bool IsIdling {
      get { return connection.IsIdling; }
    }

    public ImapSessionStatus Status {
      get { return status; }
      internal set { status = value; }
    }

    public ImapMailbox SelectedMailbox {
      get { return selectedMailbox; }
      internal set { selectedMailbox = value; }
    }

    public int Timeout {
      get { return timeout; }
      set { timeout = value; }
    }

    private ImapSession(ImapConnection connection)
    {
      this.connection = connection;

      id = connection.Id;
      status = ImapSessionStatus.NotConnected;
    }

    ~ImapSession()
    {
    }

    internal void Close()
    {
      if (connection == null)
        return;

      Trace.Info("CID:{0} session closing", id);

      connection.Disconnect();

      status = ImapSessionStatus.NotConnected;

      connection = null;

      Trace.Info("CID:{0} session closed", id);
    }

    static internal ImapSession Start(ImapUri hostUri, ImapCreateAuthenticatedStreamCallback createAuthStreamCallback)
    {
      try {
        hostUri = new ImapUri(hostUri.GetLeftPart(UriPartial.Authority));

        var connection = ImapConnection.Connect(hostUri, createAuthStreamCallback);

        var session = new ImapSession(connection);

        Trace.Info("CID:{0} session started", session.Id);

        return session;
      }
      catch (Exception ex) {
        Trace.Log(ex);

        return null;
      }
    }

    public override string ToString()
    {
      return string.Format("{{Status={0}, Connection={1}}}",
                           status,
                           connection);
    }

    private ImapConnection connection;
    private readonly int id = 0;
    private ImapSessionStatus status;
    private ImapMailbox selectedMailbox = null;
    private int timeout = 32 * 1000;
  }
}
