using System;
using System.Collections.Generic;

using Smdn.Protocols.Imap4.Protocol;

namespace Smdn.Protocols.Imap4.Transaction.Transactions {
  internal class CapabilityTransaction : TransactionBase {

    // ProcessCapability
    //   ->WaitForCapability
    // WaitForCapability
    //   x Conclude

    public CapabilityTransaction(ImapConnection connection, ImapRequest request)
      : base(connection, request)
    {
    }

    protected override ProcessTransactionDelegate Reset()
    {
      return ProcessCapability;
    }

    private void ProcessCapability()
    {
      lastTag = Send("capability");

      Transit(WaitForCapability);
    }

    private void WaitForCapability()
    {
      ImapProtocolResponse response;

      if (!Receive(out response))
        return;

      if (response is ImapDataResponse) {
        if (!ImapDataResponseConverter.FromCapabilityResponse(response as ImapDataResponse, out capability))
          Conclude(new ImapResponse(ImapTransactionStatus.Forbidden, "capability comprehension error occurred"));
      }
      else if (response is ImapStatusResponse) {
        var status = (response as ImapStatusResponse);

        if (status.Tag != lastTag)
          return;

        if ((capability.KnownCapabilities & ImapKnownCapabilities.Imap4Rev1) == 0) {
          Conclude(new ImapResponse(ImapTransactionStatus.BadRequest, "server doesn't support IMAP4rev1"));
        }
        else if (status.Status == ImapProtocolStatus.OK) {
          Connection.ServerInfo.Capability = capability;

          Conclude(new ImapCapabilityResponse(ImapTransactionStatus.Ok,
                                              capability,
                                              "comprehended server capabilities"));
        }
        else {
          ConcludeError(status);
        }
      }
    }

    private string lastTag = string.Empty;
    private ImapCapability capability = null;
  }
}
