using System;

using Smdn.Protocols.Imap4.Protocol;

namespace Smdn.Protocols.Imap4.Transaction.Transactions {
  internal class NoOpTransaction : TransactionBase {
    // ProcessNoOp
    //   ->WaitForNoOp
    // WaitForNoOp
    //   x Conclude

    public NoOpTransaction(ImapConnection connection, ImapRequest request)
      : base(connection, request)
    {
    }

    protected override ProcessTransactionDelegate Reset()
    {
      return ProcessNoOp;
    }

    private void ProcessNoOp()
    {
      lastTag = Send("noop");

      Transit(WaitForNoOp);
    }

    private void WaitForNoOp()
    {
      ImapProtocolResponse response;

      if (!Receive(out response))
        return;

      if (response is ImapStatusResponse) {
        var status = response as ImapStatusResponse;

        if (status.Tag != lastTag)
          return;

        if (status.Status == ImapProtocolStatus.OK) {
          Conclude(new ImapResponse(ImapTransactionStatus.Ok, "no operation"));
        }
        else {
          ConcludeError(status);
        }
      }
    }

    private string lastTag;
  }
}
